C--------------------------------------------------------------------
C  Copyright (C) 2002,  Space Science and Engineering Center, 
C  University C  of Wisconsin-Madison, Madison WI.
C      
C  This program is free software; you can redistribute it 
C  and/or modify it under the terms of the GNU General 
C  Public License as published by the Free Software Foundation; 
C  either version 2 of the License, or (at your option) any 
C  later version.
C
C  This program is distributed in the hope that it will be 
C  useful, but WITHOUT ANY WARRANTY; without even the implied 
C  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
C  See the  GNU General Public License for more details.
C
C  You should have received a copy of the GNU General Public 
C  License along with this program; if not, write to the Free 
C  Software Foundation, Inc., 59 Temple Place, Suite 330, 
C  Boston, MA  02111-1307 USA
C--------------------------------------------------------------------
C
C
      program cloudmask

      implicit none 
      save

C----------------------------------------------------------------------
C!F77
C
C                 USAGE:  cloudmask.exe cloudmask.cfg
C
C!DESCRIPTION:   Calculate the probability that a single MODIS pixel is 
C                unobstructed. This version has been modified for use
C                as part of the International MODIS/AIRS Processing
C                Package (IMAPP). 
C
C                MODIS radiances and reflectances are passed into 
C                processing routines based upon time of day, latitude, 
C                and surface to determine a confidence level of 
C                onobstructed. It outputs the results into a 48 bit file
C                containing the final confidence plus information on 
C                individual test results. This version uses a sliding 
C                3x3 box approach to regional processing, to clarify 
C                scenes of lower confidence.
C
C!INPUT PARAMETERS:
C  The cloudmask.cfg file will contain a list of all inputs.
C
C  Required Inputs Are:  MODIS 1 km flat file radiances and reflectances
C                        MODIS flat file geolocation parameters
C                        MODIS flat file Metadata information (Day/Night
C                              flag, etc.)
C                        Global 1 km Olson ecosystem file (static)
C           
C  Optional Inputs Are:  MODIS 250 M file reflectances (mandatory for
C                              production of 250 m mask)
C                        Global SSM/I Sea Ice File 
C                        NISE northern and southern hemisphere global 
C                              snow and ice extent files.  
C            NOTE:  The snow and ice ancillary data are 
C                   strongly recommended for use in regions
C                   of snow and ice cover.
C                        
c       The 19 MODIS channels used in this processing are:
c               1 -   .565 microns
c               2 -   .659
c               3 -   .865
c               4 -   .936
c               5 -  1.240
c               6 -  1.375
c               7 -  1.640
c               8 -  2.130
c               9 -  3.750
c               10 - 3.959
c               11 - 4.050
c               12 - 6.715
c               13 - 8.550
c               14 - 11.030
c               15 - 12.020
c               16 - 13.335
c               17 - 13.635
c               18 - 13.935
c               19 - 14.235
c
c!Output Parameters:
c  OUTPUT:  MODIS 48 bit cloud mask and Header file
c           MODIS Debug file.
c 
c!References and Credits:
c See Cloud Mask ATBD-MOD-06.  
c 
c 
c!END
c----------------------------------------------------------------------

c --- include files
      include 			'global.inc'
      include                   'platform_name.inc'

c --- external functions
c     integer 			Delete_CommonTempFiles_Atmos
      
c --- external subroutines 
c     external 			Get_Core_Metadata
c     external 			Get_Global_Metadata
c     external			store_context
c     external			store_meta
c     external			chk_lin_edge
c     external			chk_ele_edge
c     external			pxinit
c     external			get_pxldat
c     external			check_reg_uniformity
c     external			polar_day
c     external			polar_nite
c     external			land_day
c     external			land_nite
c     external			water_day
c     external			water_nite
c     external			shadows
c     external			noncld_obs_chk
c     external			thin_ci_chk_ir
c     external			proc_path
c     external			set_unused_bits
c     external			get_stats
c     external			set_confdnc
c     external			set_quality_A
c     external			fill_bit_line
c     external			reorder_array
c     external			stats_out
c     external			Prepare_ECS_Metadata
c     external			file_close
c     external			setup_create_hdf
c     external			Set_Runtime_Meta
c     external			get_geo_copy
c     external			get_crad_dat


      real			pcbad
      real			pcn1s
      real			pcn2s
      real			pcn3s
      real			pcn4s
      real			pcnnc
      real			pcnns
      real			pcnnd
      real			pcnnt
      real			pcnng
      real			pcnni
      real			pcnnl
      real			pcnnw
      real			pcnnr
      real			pcnnv
      real			pcnncr
      real			pcnncd
      real			pcnu
      real			pcnn
      real			pcnz
      real			pcna
      real			pcne


c --- external functions
      integer			get_rp
      integer			get_core_metadata
      integer	 		get_swath_metadata
      integer			get_L1b
      integer			get_angles
      integer			get_anc_data
      integer			put_mod35
      integer                   iargc

c --- external subroutines
      external			file_open
      external 			init_input
      external			message
      external 			init_meta
      external 			chk_input
      external 			init_var
      external			clear_radiance

c --- internal variables
      character*1 		scan_type
      character*1  		s_type(NLCNTX)
      character*4		interleave_1km
      character*4               interleave_250m
      character*4		interleave_geo
      character*80	 	bandnames_1km(INBAND)
      character*80		bandunits_1km(INBAND)
      character*80	 	bandnames_250m(2)
      character*80		bandunits_250m(2)
      character*80	 	bandnames_geo(20)
      character*80		bandunits_geo(20)
      character*80 		thresholds_version
      character*132             cfgname
      character*512		l1b_1km_hdr
      character*512		l1b_250m_hdr
      character*512		l1b_geo_hdr
      byte 			eco_type 
      byte 			qa_bits(10)
      byte 			mask_array(NPIXEL,6)
      byte 			qa_array(NPIXEL,10)
      byte 			contx_eco(NPIXEL,NLCNTX)
      byte 			v250_band(NX,NY,VIS_BAND)
      byte 			v1km_band(NX,NY,INBAND)
      byte 			cube_eco(NPIXEL,SCANS_CUBE)
      byte 			testbits(6)
      integer 			cube
      integer			index_day(NC_MAX)
      integer			index_night(NC_MAX) 
      integer			prev_cube
      integer			line
      integer			line_count
      integer			modfil_L1B_1km(MODFILLEN)
      integer			modfil_L1B_250m(MODFILLEN)
      integer			modfil_mod35(MODFILLEN)
      integer			modfil_Geo(MODFILLEN)
      integer			modfil_clr_out(MODFILLEN)
      integer                   buf_size(2)
      integer                   buf_geo_size(2)
      integer                   buf_anc_size(2)
      integer			eco_1km_lun
      integer			eco_10min_lun
      integer			l1b_1km_lun
      integer			l1b_250m_lun
      integer			l1b_geo_lun
      integer			l1b_scan_lun
      integer			lsf
      integer			anc_met_lun
      integer			anc_nisen_lun
      integer			anc_nises_lun
      integer			anc_icec_lun
      integer			anc_ozone_lun
      integer			anc_sst_lun
      integer			anc_threshold_lun
      integer			mask_lun
      integer			qa_lun
      integer			hdr_lun
      integer			qahdr_lun
      integer 			outrec
      integer			geo_flag(5)
      integer			status
      integer			nscans
      integer			max_pixel
      integer			lines_in_edge
      integer			pixels_in_edge
      integer 			beg_lin
      integer 			nlins
      integer 			beg_ele
      integer 			neles
      integer 			maxlin
      integer 			maxele
      integer			max_count
      integer			max5km_ele
      integer			max5km_lin
      integer			mirror_side
      integer			nu
      integer			nn
      integer			nz
      integer			na
      integer			ne
      integer			nc
      integer			num_bad
      integer			nl
      integer			nw
      integer			nr
      integer			nv
      integer			n250
      integer			ncr
      integer			ncd
      integer			n1sm
      integer			n2sm
      integer			n3sm
      integer			n4sm
      integer			nbad_1km
      integer			nbad_250
      integer			nbands
      integer			nd
      integer			ng
      integer			ni
      integer			nmtests
      integer			npix
      integer			no
      integer			ns
      integer			nt
      integer			I_ind
      integer			ilin
      integer			mlin
      integer			klin
      integer			nadir
      integer                   num_arg
      integer			ncells_day
      integer			ncells_night
      integer			scan_pixels
      integer			scan_number
      integer			scan_flag
      integer			datatype_1km, datatype_250m
      integer			datatype_geo
      integer			resolution_1km, resolution_250m
      integer			resolution_geo
      integer			offset_1km, offset_250m
      integer			offset_geo
      integer			samples_1km, samples_250m
      integer			samples_geo
      integer			lines_1km, lines_250m
      integer			lines_geo
      integer			bands_1km, bands_250m
      integer			bands_geo
      integer			s_pixels(NLCNTX)
      integer			n_nadir(NLCNTX)
      integer			contx_topog(NPIXEL,NLCNTX)
      integer			contx_snow(NPIXEL,NLCNTX)
      integer			cube_topog(NPIXEL,SCANS_CUBE) 
      integer			cube_snow(NPIXEL,SCANS_CUBE)
      real			clear_day(NSTATS,CR_BAND,NC_MAX)
      real 			confdnc
      real			vis250_band(NX,NY,VIS_BAND)
      real			rad_band(NX,NY,INBAND)
      real			clear_night(NSTATS,CR_BAND,NC_MAX)
      real			contx_lat(NPIXEL,NLCNTX)
      real			contx_lon(NPIXEL,NLCNTX)
      real			contx_elev(NPIXEL,NLCNTX)
      real			contx_vzen(NPIXEL,NLCNTX)
      real			contx_szen(NPIXEL,NLCNTX)
      real			contx_sfctmp(NPIXEL,NLCNTX)
      real			contx_msl(NPIXEL,NLCNTX)
      real			contx_ugrd(NPIXEL,NLCNTX)
      real			contx_vgrd(NPIXEL,NLCNTX)
      real			contx_rel(NPIXEL,NLCNTX)
      real			contx_pw(NPIXEL,NLCNTX)
      real			contx_ice(NPIXEL,NLCNTX)
      real			pxldat(INBAND)
      real 			precip_water
      real			crad_dat(CR_BAND)
      real 			pxl250(VIS_BAND,NUM250_PER_1KM)
      real 			pmsl
      real 			plat
      real 			plon
      real			indat(NPIXEL,NLCNTX,INBAND)
      real			indat_rad(NPIXEL,NLCNTX,INBAND)
      real			v250_dat(NX*4,NLCNTX,VIS_BAND)
      real			max_sol
      real			min_sol
      real			error_1km, error_250m     
      real			error_geo     
      real			tbadj
      real			rlat(NPIXEL,SCANS_CUBE)
      real			rlon(NPIXEL,SCANS_CUBE)
      real			view_zen(NPIXEL,SCANS_CUBE)
      real			elev(NPIXEL,SCANS_CUBE)
      real			solar_zen(NPIXEL,SCANS_CUBE)
      real			rel_angle(NPIXEL,SCANS_CUBE)
      real			cube_pw(NPIXEL,SCANS_CUBE)
      real			cube_ice(NPIXEL,SCANS_CUBE)
      real			cube_sfctmp(NPIXEL,SCANS_CUBE)
      real			cube_msl(NPIXEL,SCANS_CUBE)
      real			cube_ugrd(NPIXEL,SCANS_CUBE)
      real			cube_vgrd(NPIXEL,SCANS_CUBE)
      real                      scan_confdnc(npixel)
      real 			sfctmp
      real 			u_wind
      real 			v_wind
      real 			vza
      logical 			antarctic
      logical			bad_geo
      logical			bad_value
      logical			cirrus_ir
      logical			cirrus_vis
      logical			coast
      logical                   day
      logical			desert
      logical                   ele_edge
      logical 			hi_elev
      logical			ice
      logical 			map_ice
      logical 			map_snow
      logical                   night
      logical			yes_250
      logical                   land
      logical                   line_edge
      logical                   polar
      logical			process
      logical 			sg_bad_data
      logical 			sh_ocean
      logical			shadow
      logical			smoke
      logical                   snglnt
      logical			snow
      logical			uniform
      logical			visusd
      logical			vrused
      logical                   water
      logical                   clrrad_flag

c ... New logical variables for 250 m data.
      logical scan_ice(npixel),scan_snow(npixel),scan_desert(npixel),
     +        scan_coast(npixel),scan_land(npixel),
     +        scan_snglnt(npixel),cm1km(num250_per_1km),
     +        ice_250(num250_per_1km),snow_250(num250_per_1km),
     +        desert_250(num250_per_1km),coast_250(num250_per_1km),
     +        land_250(num250_per_1km),
     +        snglnt_250(num250_per_1km),scan_visusd(npixel),
     +        scan_process(npixel),day_250(num250_per_1km),
     +        proc_250(num250_per_1km),visusd_250(num250_per_1km),
     +        qa_250(num250_per_1km),scan_day(npixel)

c --- Common Block for debugging code 
      integer			debug		! debug flag
      integer			h_output	! output file handle
      common / bug / debug, h_output

c --- data 
      data outrec/0/                    	! Initial output record number

c --- Write start message
      WRITE(*,*) 'Direct Broadcast Cloud Mask Initiated.'

c --- Get input argument of configuration file containing source 
c ---  input and output file information.

c --- Check number of arguments
      num_arg = iargc()
      if (num_arg .ne. 1) then
        print *, 'Usage: cloudmask.exe cfgname'
        print *, 'where'
        print *, 'cfgname:  name of cloud mask configuration file'
        call exit(-1)
      endif

c ... Extract arguments
      call getarg(1, cfgname)


c --- Extract debug value and processing bounds from context file 
      status = get_rp( 
     &                 cfgname,		       	! Config file name
     &                 l1b_1km_hdr,		! l1b 1km header 
     &                 l1b_250m_hdr,		! l1b 250m header 
     &                 l1b_geo_hdr,		! l1b geo header
     &                 debug, 		       	! debug flag 
     &                 beg_lin, 	       	! beginning line number
     &                 nlins, 		       	! number of lines
     &                 beg_ele, 	       	! beginning element
     &                 neles                 	! swath width
     &               )
      if( status.lt.0 ) then
         call message( 'cloudmask:',
     &                 'FAILED - read processing bounds',
     &                 0, 3 )
      endif

c --- Hard code Terra into platform name for now. Need new thresholds
c ---  after Aqua launch.
      platform_name = 'Terra'

c --- Hard code clear radiance file use.  Currently not implemented
c --- in DB code
      clrrad_flag = .false.

c --- Intialize variables needed for inputs and outputs
      call init_input(
     &                 modfil_L1B_1km,		! 1km data array	
     &                 modfil_L1B_250m,		! 250m data array
     &                 modfil_mod35,		! cloudmask array
     &                 modfil_Geo,		! geo array
     &                 modfil_clr_out, 		! clear radiance array
     &                 buf_size,		! size of the 1km array
     &                 buf_geo_size,		! size of geo array
     &                 buf_anc_size,		! size of ancillary file array
     &                 qa_bits,			! output quality bits
     &                 mask_array,		! output cloud mask array
     &                 qa_array		        ! output quality control array
     &               )


c --- Open all input data files 
      call file_open(
     &                 cfgname,		       	! Config file name
     &                 neles,                	! swath width
     &                 eco_1km_lun,		! 1km ecosystem
     &                 eco_10min_lun,     	! 10min ecosystem
     &                 l1b_250m_lun,      	! 250m L1B data
     &                 l1b_1km_lun,	      	! 1km L1B data
     &                 l1b_geo_lun,       	! 1km Geo data
     &                 l1b_scan_lun,         	! 1km Scan metadata
     &                 anc_met_lun,   		! land sea mask
     &                 anc_nisen_lun,     	! NH ice/snow
     &                 anc_nises_lun,     	! SH ice/snow
     &                 anc_icec_lun,    	! sea ice
     &                 anc_ozone_lun,        	! ozone 
     &                 anc_sst_lun,          	! sea surface temperature
     &                 anc_threshold_lun, 	! thresholds 
     &                 mask_lun,    		! cloud mask
     &                 hdr_lun,                 ! cloud mask header
     &                 qahdr_lun,               ! cloud mask qa header
     &                 qa_lun,			! quality control
     &                 h_output,         	! debug
     &                 yes_250	      		! 250m data available
     &               )

c --- set bits used in future expansion
      call set_qa_bit( qa_bits, 56)		! Unknown file
      call set_qa_bit( qa_bits, 57)		! Unknown file
      call set_qa_bit( qa_bits, 72) 		! Digital Elevation


c --- Initialize variables needed to perform metadata extraction
      call init_meta(
     &                nscans,			! number of scans
     &                max_pixel,		! maximum number of EV's per line
     &                lines_in_edge,		! line boundary for mask
     &                pixels_in_edge		! element boundary for mask
     &              )

c --- Find bounds of granule from reading Core Metadata
      status = get_core_metadata( 
     &         yes_250,
     &         l1b_1km_hdr,    l1b_250m_hdr,    l1b_geo_hdr,
     &         nscans,
     &         max_pixel,
     &         datatype_1km,   datatype_250m,   datatype_geo,
     &         interleave_1km, interleave_250m, interleave_geo,
     &         resolution_1km, resolution_250m, resolution_geo,
     &         offset_1km,     offset_250m,     offset_geo,
     &         samples_1km,    samples_250m,    samples_geo,
     &         lines_1km,      lines_250m,      lines_geo,
     &         error_1km,      error_250m,      error_geo,
     &         bands_1km,      bands_250m,      bands_geo,
     &         bandnames_1km,  bandnames_250m,  bandnames_geo,
     &         bandunits_1km,  bandunits_250m,  bandunits_geo 
     &                         )
      if( status.lt.0 ) then
         call message( 'cloudmask:',
     &                 'FAILED - get core metadata',
     &                 0, 3 )
      endif


c ... check consistency of input information.
      call chk_input(
     &                max_pixel,		! maximum pixel count	
     &                nscans,			! total scans
     &                lines_in_edge,		! line boundary dimension
     &                beg_lin,			! beginning processing line
     &                nlins,			! number or processing lines
     &                beg_ele,			! beginning processing elem
     &                neles,			! number of processing elems
     &                maxele,			! maximum elem
     &                maxlin,			! maximum line
     &                max_count,		! maximum input buffer size
     &                max5km_lin,		! maximum 5km line size
     &                max5km_ele		! maximum 5km elem size
     &              )

c --- Read thresholds from parameter file
      call thresholds_read( thresholds_version )

              
c --- Finally, initialize the granule based processing variables
      call init_var(
     &              cube,         pxldat,       crad_dat, 
     &              pxl250,       indat,        indat_rad,
     &              v250_dat,     contx_lat,    contx_lon, 
     &              contx_elev,   contx_vzen,   contx_szen,
     &              contx_rel,    contx_pw,     contx_eco,
     &              contx_topog,  contx_ice,    contx_snow, 
     &              contx_sfctmp, contx_msl,    contx_ugrd, 
     &              contx_vgrd,   s_type,       s_pixels,
     &              n_nadir,      nadir,        scan_pixels,
     &              scan_number,  scan_type,    max_sol,
     &              min_sol,      ilin,         mlin, 
     &              klin,         I_ind,        prev_cube
     &             )



C-----------------------------------------------------------------------
c                          SCAN LINE LOOP
c
c     Monitor the number of lines ('NLCNTX') needed to fill a context 
c  region with the variable 'mlin'.  Determine cloud mask at the 
c  center pixel of each context region. 
c    
c     For lines and pixels with incomplete context regions at the 
c  boundaries of our processing region, determine mask, but do
c  not use spatial variability routines.
C-----------------------------------------------------------------------

c --- main processing loop
      do 100 line = beg_lin , max_count
         ilin = ilin + 1
         line_count = beg_lin - 1 + ilin
         I_ind = I_ind + 1
         cube = (line_count-1)/10 + 1
         mlin = mlin + 1


c ------ get new cube and geo info every cube (10 lines).
         if( cube.ne.prev_cube .and. ilin.le.maxlin ) then

            WRITE(*,*) '*** Processing Scan # =', cube

            prev_cube = cube
            I_ind = MOD(line_count-1,10) + 1 


c --------- Extract scan based metadata
            status = get_swath_metadata(
     &                                  l1b_scan_lun,
     &                                  cube,
     &                                  scan_flag,
     &                                  mirror_side
     &                                 )
            if( status.lt.0 ) then
               call message( 'cloudmask:',
     &                       'FAILED - get swath metadata for scan=',
     &                       cube, 3 )
            endif

c --------- equate max_pixel with scan_pixels
            scan_pixels = max_pixel

c --------- convert the scan flag to scan type
            scan_type = ' '
            if( scan_flag.eq.1 ) then 
               scan_type = 'D'
            elseif ( scan_flag.eq.0 ) then 
               scan_type = 'N'
            endif

c --------- Extract radiance and reflectance information
            status = get_L1b(
     &                        l1b_1km_lun,    l1b_250m_lun,
     &                        datatype_1km,   datatype_250m, 
     &                        interleave_1km, interleave_250m,
     &                        resolution_1km, resolution_250m,
     &                        offset_1km,     offset_250m,
     &                        samples_1km,    samples_250m,
     &                        lines_1km,      lines_250m,
     &                        error_1km,      error_250m,
     &                        bands_1km,      bands_250m,
     &                        bandnames_1km,  bandnames_250m,
     &                        bandunits_1km,  bandunits_250m,
     &                        yes_250,
     &                        cube,
     &                        buf_size,
     &                        scan_type,
     &                        v250_band, 
     &                        v1km_band, 
     &                        vis250_band,
     &                        rad_band
     &                      )
            if( status.lt.0 ) then
               call message( 'cloudmask:',
     &                       'FAILED - get L1b data',
     &                       0, 3 )
            endif

c --------- Extract the geolocation file variables 

            status = get_angles(
     &                           cube,
     &                           scan_type,
     &                           max_sol,
     &                           min_sol,
     &                           l1b_geo_lun,
     &                           datatype_geo,
     &                           interleave_geo,
     &                           resolution_geo,
     &                           error_geo,
     &                           offset_geo,
     &                           samples_geo,
     &                           lines_geo,
     &                           bands_geo,
     &                           bandnames_geo,
     &                           bandunits_geo,
     &                           rlat,
     &                           rlon,
     &                           elev,
     &                           view_zen,
     &                           solar_zen,
     &                           cube_topog,
     &                           rel_angle
     &                         )

            if( status.lt.0 ) then
               call message( 'cloudmask:',
     &                       'FAILED - get angle data',
     &                       0, 3 )
            endif
        
c --------- Extract the ancillary data
            status = get_anc_data(
     &                             anc_met_lun,
     &                             anc_ozone_lun,
     &                             anc_icec_lun,
     &                             anc_sst_lun,
     &                             anc_nisen_lun,
     &                             anc_nises_lun,
     &                             eco_1km_lun,
     &                             eco_10min_lun,
     &                             cube,
     &                             buf_anc_size,
     &                             max_pixel,
     &                             rlat,
     &                             rlon,
     &                             cube_pw,
     &                             cube_eco,
     &                             cube_ice,
     &                             cube_snow,
     &                             cube_sfctmp,
     &                             cube_msl,
     &                             cube_ugrd,
     &                             cube_vgrd,
     &                             qa_bits
     &                           )
            if( status.lt.0 ) then
               call message( 'cloudmask:',
     &                       'FAILED - get ancillary data',
     &                       0, 3 )
            endif


            I_ind = MOD(line_count-1,10) + 1 


         endif




C-----------------------------------------------------------------------
c                            Create a CONTEXT
c
c         read scan line radiance and reflectance data, and if necessary
c         convert to brightness temperature for use in threshold testing
C-----------------------------------------------------------------------

         if (ilin .le. maxlin) then

            call store_context(
     &         mlin,I_ind,maxele,rlat,rlon,elev,
     &         view_zen,solar_zen,rel_angle,v250_band,
     &         vis250_band,rad_band,v1km_band,indat,
     &         indat_rad,v250_dat,cube_pw,cube_eco,
     &         cube_topog,cube_ice,cube_snow,cube_sfctmp,
     &         cube_msl,cube_ugrd,cube_vgrd,contx_lat,
     &         contx_lon,contx_elev,contx_vzen,contx_szen,
     &         contx_rel,contx_pw,contx_eco,contx_topog,
     &         contx_ice,contx_snow,contx_sfctmp,
     &         contx_msl,contx_ugrd,contx_vgrd)


            call store_meta(mlin,scan_type,scan_pixels,nadir,
     &                      s_type,s_pixels,n_nadir)


         endif 




C-----------------------------------------------------------------------
C                          PROCESSING SECTION
c
c        The Cloud Mask is processed pixel by pixel using a sliding
c        box approach. Regional context data is stored for use
c        in uniformity tests for uncertain pixels.  The edge of 
c        the area in which you are processing, (outline of region)
c        will be processed but will not include uniformity tests.
C----------------------------------------------------------------------


c ------ Check if current line is a part of the border
         call chk_lin_edge(klin,maxlin,lines_in_edge,line_edge)
 
         if( mlin.eq.NLCNTX .or. line_edge) then

            klin = klin + 1

            Do 300 nc = beg_ele , maxele
c               Check if current element is a part of the border
                call chk_ele_edge(nc,s_pixels,pixels_in_edge,klin,
     +                            maxlin,lines_in_edge,ele_edge)

c               Initialize regional variables
                call pxinit(testbits,qa_bits,geo_flag,precip_water,vza,
     +                  sfctmp,pmsl,u_wind,v_wind,plat,plon,lsf,polar,
     +                  day,night,land,water,coast,snglnt,visusd,vrused,
     +                  snow,ice,desert,bad_value,bad_geo,uniform,
     +                  shadow,smoke,cirrus_ir,cirrus_vis,nmtests,
     +                  nbands,nbad_1km,nbad_250,hi_elev,antarctic,
     +                  sh_ocean,sg_bad_data,map_ice,map_snow)

c               Get pixel values for processing variables
                call get_pxldat(indat,lines_in_edge,contx_lat,
     +                          contx_lon,contx_vzen,contx_szen,
     +                          contx_elev,contx_rel,contx_pw,contx_eco,
     +                          contx_topog,contx_ice,contx_snow,
     +                          contx_sfctmp,contx_msl,contx_ugrd,
     +                          contx_vgrd,s_type,klin,beg_lin,maxlin,
     +                          nc,nbands,nbad_1km,pxldat,
     +                          plat,plon,precip_water,vza,
     +                          sfctmp,pmsl,u_wind,v_wind,eco_type,
     +                          lsf,geo_flag,polar,day,night,land,water,
     +                          coast,snglnt,visusd,ice,vrused,desert,
     +                          snow,bad_value,bad_geo,hi_elev,tbadj,
     +                          antarctic,sh_ocean,sg_bad_data,map_ice,
     +                          map_snow)


c               Check for consistency of variables within processing
c               region

                call check_reg_uniformity(line_edge,ele_edge,
     +                                   contx_szen,contx_eco,
     +                                   contx_topog,contx_snow,
     +                                   s_type,n_nadir,nc,
     +                                   eco_type,day,night,land,
     +                                   water,coast,snow,ice,uniform)

c               Decision to process or not to process this pixel.
                if(sg_bad_data .and. water .and. snglnt) then
                  process = .false.
                else
                  process = .true.
                end if

                if(process) then

c                 Decision tree for processing paths 
                  if (polar) then

c                   First polar processing

                    if (day) then
c                     Daytime processing
                      call polar_day(pxldat,vza,snglnt,
     +                               visusd,vrused,cirrus_vis,
     +                               land,ice,snow,desert,coast,
     +                               eco_type,uniform,hi_elev,nc,indat,
     +                               nmtests,testbits,tbadj,antarctic,
     +                               sh_ocean,qa_bits,confdnc)
                    else
c                     Nighttime processing
                      call polar_nite(pxldat,vza,land,ice,snow,nmtests,
     +                                testbits,uniform,indat,nc,maxele,
     +                                line_edge,klin,qa_bits,confdnc)
                    endif
                     
                  
c --------------- Primarily land surface.
                  else if (land) then

c ------------------ Daytime processing.
                     if (day) then
                      call land_day(pxldat,vza,visusd,vrused,
     +                              cirrus_vis,desert,coast,snow,
     +                              ice,hi_elev,tbadj,eco_type,testbits,
     +                              qa_bits,nmtests,confdnc)

c ------------------ Nighttime processing.
                     else
                        call land_nite(pxldat,vza,ice,snow,nmtests,
     &                                 testbits,qa_bits,confdnc)

                    endif

                  else if (water) then
c                   Primarily water surface.

                    if (day) then
c                     Daytime processing.
                      call water_day(pxldat,vza,snglnt,visusd,
     +                               cirrus_vis,hi_elev,uniform,ice,
     +                               sh_ocean,indat,nc,nmtests,testbits,
     +                               qa_bits,confdnc)
                    else
c                     Nighttime processing.
                      call water_nite(pxldat,vza,uniform,ice,indat,
     +                                nc,maxele,line_edge,klin,nmtests,
     +                                testbits,qa_bits,confdnc)
                    end if

                  end if

c                 Test for shadows,if necessary. Set bit to indicate 
c                 no shadow was found.
                  if(.not.water .and. .not.coast .and. day .and. 
     +               .not.polar .and. confdnc.ge.0.66) then
                    call shadows(pxldat,shadow,visusd,qa_bits)
                  end if

c                 Test for possible non-cloud obstruction.
                  if(land .and. day .and. .not. polar) then
                      call noncld_obs_chk(pxldat,smoke,qa_bits,testbits,
     +                                    confdnc)
                  end if

c                 Test of thin cirrus in the infrared
                  if (.not.snow.and..not.ice.and.confdnc.ge.0.95) then
                    call thin_ci_chk_ir(pxldat,vza,cirrus_ir,
     +                                  qa_bits,testbits)
                  endif

c                 Fill clear radiance array if high confidence clear
                  if (confdnc .gt. 0.95 .and. .not. bad_geo .and.
     +                clrrad_flag) then

                    if( .not. (water .and. snglnt)) then
                      call get_crad_dat(indat_rad,lines_in_edge,klin,
     +                                  beg_lin,maxlin,nc,crad_dat)

                      call clear_radiance(
     &                                     plat,
     &                                     plon,
     &                                     day,
     &                                     NSTATS,
     &                                     CR_BAND,
     &                                     NC_MAX,
     &                                     crad_dat,
     &                                     ncells_day,
     &                                     ncells_night,
     &                                     clear_day,
     &                                     clear_night,
     &                                     index_day,
     &                                     index_night
     &                                   )

                    endif


                  endif

                end if

c               Set bits which indicate processing path through 
c               algorithm. 
                call proc_path( 
     &                          water,
     &                          land,
     &                          day,
     &                          ice,
     &                          snow,
     &                          snglnt,
     &                          coast,
     &                          desert,
     &                          smoke,
     &                          shadow,
     &                          testbits
     &                        )

c               Set the bits which are not used in output array.
                call set_unused_bits(testbits)

c               Get cloud mask statistics
                call get_stats(day,land,water,snglnt,snow,coast,desert,
     +                         ice,shadow,smoke,cirrus_ir,cirrus_vis,
     +                         confdnc,nmtests,nbands,bad_value,bad_geo,
     +                         geo_flag,no,ns,nd,nt,ng,
     +                         ni,nl,nw,nr,nv,nu,nn,nz,na,
     +                         ne,npix,n1sm,n2sm,n3sm,n4sm,num_bad)


c               Set cloud mask quality bit flags
                call set_confdnc(confdnc,testbits)

c               Set remaining QA bits 
                call set_quality_A(nmtests,nbands,lsf,
     &                             eco_1km_lun,qa_bits)

c               Store element in line array 
                call fill_bit_line(
     &                              nc,
     &                              nmtests,
     &                              nbands,
     &                              bad_value,
     &                              bad_geo,
     &                              snglnt,
     &                              desert,
     &                              testbits,
     &                              qa_bits,
     &                              mask_array,
     &                              qa_array
     &                            )

                call store_cm_scan(nc,confdnc,day,visusd,snglnt,snow,
     +                             ice,coast,desert,land,process,
     +                             scan_confdnc,scan_visusd,scan_snglnt,
     +                             scan_snow,scan_ice,scan_coast,
     +                             scan_desert,scan_land,scan_process,
     +                             scan_day)


300         continue

c            Create the 250-m cloud mask corresponding to the current
c            1-km scan line.

             do 200 nc = beg_ele, maxele

               call get_250m_data(maxlin,lines_in_edge,klin,nc,maxele,
     +                            v250_dat,scan_confdnc,scan_ice,
     +                            scan_snow,scan_desert,scan_coast,
     +                            scan_land,pxl250,cm1km,
     +                            ice_250,snow_250,desert_250,coast_250,
     +                            land_250,scan_snglnt,
     +                            snglnt_250,scan_day,scan_process,
     +                            scan_visusd,qa_array,day_250,
     +                            proc_250,visusd_250,qa_250)

               call create_250m_cm(nc,pxl250,cm1km,ice_250,snow_250,
     +                             desert_250,coast_250,land_250,
     +                             mask_array,qa_array,n250,ncr,ncd,
     +                             day_250,proc_250,visusd_250,qa_250,
     +                             snglnt_250)

  200        continue



c --------- write scanline of data into output file
            status = put_mod35(
     &                          mask_lun,
     &                          hdr_lun,
     &                          qa_lun,
     &                          qahdr_lun,
     &                          s_pixels,
     &                          maxlin,
     &                          klin,
     &                          lines_in_edge,
     &                          mask_array,
     &                          qa_array 
     &                        )



c --------- keep track of number of lines processed
            outrec = outrec + 1

c --------- Check to see if processing first lines in edge of
c           context.  Don't want to rebuffer or change counter.
            if (klin .gt. lines_in_edge) then

c ------------ if not processing in single-line mode, re-buffer data 
c              in order to add the next scan line.
               call reorder_array(contx_lat,contx_lon,contx_vzen,
     &                            contx_szen,contx_rel,contx_pw,
     &                            contx_eco,contx_topog,contx_ice,
     &                            contx_snow,contx_sfctmp,contx_msl,
     &                            contx_ugrd,contx_vgrd,contx_elev,
     &                            s_type,s_pixels,n_nadir,maxele,indat,
     &                            indat_rad,v250_dat)

c ------------ Set context counter to 2 so that when processing 
c              next line will be in right order. 
               mlin = 2
            end if 

         endif

100   continue

c     Write data set statistics to debug file.
      call stats_out(npix,num_bad,n1sm,n2sm,n3sm,n4sm,no,ns,
     +               nd,nt,ng,ni,nl,nw,nr,nv,n250,ncr,
     +               ncd,nu,nn,nz,na,ne,pcbad,pcn1s,pcn2s,pcn3s,pcn4s,
     +               pcnnc,pcnns,pcnnd,pcnnt,pcnng,pcnni,pcnnl,pcnnw,
     +               pcnnr,pcnnv,pcnncr,pcnncd,pcnu,pcnn,pcnz,pcna,
     +               pcne)


      WRITE(*,*) 'Direct Broadcast Cloud Mask Completed.'

      end
