#!/bin/csh -f

# This script is an example of how to set up processing for the 
# the DB MODIS cloud mask.

# Originally written by Liam Gumley liam.gumley@ssec.wisc.edu March, 2002.

#-------------------------------------------------------------------------------
# SETUP
#-------------------------------------------------------------------------------

# Check arguments
if ($#argv != 4) then
  echo "Usage: modis_cloudmask.csh FIL1KM FILQKM FILGEO OUTDIR"
  echo "where"
  echo "FIL1KM is MODIS L1B 1000 meter resolution HDF image file"
  echo "FILQKM is MODIS L1B 250  meter resolution HDF image file"
  echo "FILGEO is MODIS L1B 1000 meter resolution HDF geolocation file"
  echo "OUTDIR is the directory where products will be stored"
  exit(-1)
endif

# Extract file names
set FIL1KM = $argv[1]
set FILQKM = $argv[2]
set FILGEO = $argv[3]
set OUTDIR = $argv[4]

# Set root of output file name (e.g. 't1.02001.1815')
set ROOT = $FIL1KM:r
set ROOT = $ROOT:r
set ROOT = $ROOT:t

#-------------------------------------------------------------------------------
# GET ANCILLARY DATA IF NECESSARY
# 
#  If you are not processing a data set where snow and ice
#  are present, you can delete or comment out the the 
#  NISE and NCEP_ICE portions of this section.

#  Works off of ancillary data sets in the given directory.

#  All dynamic ancillary data sets are available real-time at: 
#   ftp://terra.ssec.wisc.edu/pub/terra/ancillary
#-------------------------------------------------------------------------------

# Set directory for ancillary data
set ANC_DIR = ../../ancillary/data

# Set default names for unpacked NISE files
set NISE_NORTH_BIN = ""
set NISE_SOUTH_BIN = ""

# Set directory for ancillary scripts
set ANC_SRC = ../../ancillary/src

set NISE_FILE = `find $ANC_DIR -name "NISE_SSMIF13_????????.HDFEOS" | sort | tail -1`
if ($NISE_FILE == "") then
  echo "No NISE files were found in "$ANC_DIR
  exit(1)
endif

# Point to and unpack the NISE data file
set NISE_NORTH_BIN = ${NISE_FILE:t}_NORTH.bin
set NISE_SOUTH_BIN = ${NISE_FILE:t}_SOUTH.bin
$ANC_SRC/extract_nsidc_nise.csh $NISE_FILE $NISE_NORTH_BIN $NISE_SOUTH_BIN
endif
  
# Set default name for unpacked NCEP ice file
set NCEP_ICE_BIN = ""

# Get name of correct ncep ICE file (should be only one in directory) 
set NCEP_ICE_FILE = `find $ANC_DIR -name "eng.??????" | sort | tail -1`
if ($NCEP_ICE_FILE == "") then
  echo "No NCEP ice files were found in "$ANC_DIR
  exit(1)
endif

set NCEP_ICE_BIN = ${NCEP_ICE_FILE:t}.bin
$ANC_SRC/extract_ncep_ice.csh $NCEP_ICE_FILE $NCEP_ICE_BIN

#-------------------------------------------------------------------------------
# RUN CLOUD MASK ALGORITHM
#-------------------------------------------------------------------------------

# Print start message for processing log
echo
echo ">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>"
echo "Started MODIS cloud mask processing at "`date`
echo "MODIS L1B 1KM: "$FIL1KM
echo "MODIS L1B QKM: "$FILQKM
echo "MODIS L1B GEO: "$FILGEO

# Extract flat files
echo "Started flatfile extraction at "`date`
set SRCDIR = ../../flatfile/src
set DATADIR = ../../flatfile/data
$SRCDIR/modis_extract_1km.exe $DATADIR/$FIL1KM ${ROOT}.1000m.img ${ROOT}.1000m.hdr
$SRCDIR/modis_extract_qkm.exe $DATADIR/$FILQKM ${ROOT}.250m.img  ${ROOT}.250m.hdr
$SRCDIR/modis_extract_geo.exe $DATADIR/$FILGEO ${ROOT}.geo.img   ${ROOT}.geo.hdr
$SRCDIR/modis_extract_met.exe $DATADIR/$FIL1KM ${ROOT}.met.img   ${ROOT}.met.hdr
echo "Finished flatfile extraction at "`date`

# Set output file names
set OUTFIL      = $ROOT.mod35.img
set OUTFILQA    = $ROOT.mod35qa.img
set OUTFILHDR   = $ROOT.mod35.hdr
set OUTFILQAHDR = $ROOT.mod35qa.hdr
set DEBUG       = $ROOT.mod35debug.txt

# Create a configuration file from the template
set TEMPLATE = ../cmd/cloudmask.cfg
/bin/sed \
  -e "s/FIL1KM_IMG/${ROOT}.1000m.img/g" \
  -e "s/FIL1KM_HDR/${ROOT}.1000m.hdr/g" \
  -e "s/FILQKM_IMG/${ROOT}.250m.img/g" \
  -e "s/FILQKM_HDR/${ROOT}.250m.hdr/g" \
  -e "s/FILGEO_IMG/${ROOT}.geo.img/g" \
  -e "s/FILGEO_HDR/${ROOT}.geo.hdr/g" \
  -e "s/FILMET_IMG/${ROOT}.met.img/g" \
  -e "s/FILMET_HDR/${ROOT}.met.hdr/g" \
  -e "s/NISE_NORTH/${NISE_NORTH_BIN}/g" \
  -e "s/NISE_SOUTH/${NISE_SOUTH_BIN}/g" \
  -e "s/NCEP_ICE/${NCEP_ICE_BIN}/g" \
  -e "s/MOD35.img/${OUTFIL}/g" \
  -e "s/MOD35QA.img/${OUTFILQA}/g" \
  -e "s/MOD35.hdr/${OUTFILHDR}/g" \
  -e "s/MOD35QA.hdr/${OUTFILQAHDR}/g" \
  -e "s/debug.dat/${DEBUG}/g" \
  $TEMPLATE > cloudmask.cfg

# Run the cloud mask
set SRCDIR = ../cmd
$SRCDIR/cloudmask.exe cloudmask.cfg

# Rename the configuration file
/bin/mv cloudmask.cfg $ROOT.mod35.cfg

# Move the output files to the product directory
/bin/mv $OUTFIL $OUTFILQA $OUTFILHDR $OUTFILQAHDR $DEBUG $ROOT.mod35.cfg $OUTDIR

#-------------------------------------------------------------------------------
# CLEANUP AND EXIT
#-------------------------------------------------------------------------------

# Print end message for processing log
echo
echo ">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>"
echo "Finished MODIS cloud mask product processing at "`date`

# Exit gracefully
exit(0)
