C--------------------------------------------------------------------
C  Copyright (C) 2002,  Space Science and Engineering Center, 
C  University C  of Wisconsin-Madison, Madison WI.
C      
C  This program is free software; you can redistribute it 
C  and/or modify it under the terms of the GNU General 
C  Public License as published by the Free Software Foundation; 
C  either version 2 of the License, or (at your option) any 
C  later version.
C
C  This program is distributed in the hope that it will be 
C  useful, but WITHOUT ANY WARRANTY; without even the implied 
C  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
C  See the  GNU General Public License for more details.
C
C  You should have received a copy of the GNU General Public 
C  License along with this program; if not, write to the Free 
C  Software Foundation, Inc., 59 Temple Place, Suite 330, 
C  Boston, MA  02111-1307 USA
C--------------------------------------------------------------------
C
C
      integer function  get_L1b(
     &                           lun_1km,        lun_250m,
     &                           datatype_1km,   datatype_250m,
     &                           interleave_1km, interleave_250m,
     &                           resolution_1km, resolution_250m,
     &                           offset_1km,     offset_250m,
     &                           samples_1km,    samples_250m,
     &                           lines_1km,      lines_250m,
     &                           error_1km,      error_250m,
     &                           bands_1km,      bands_250m,
     &                           bandnames_1km,  bandnames_250m,
     &                           bandunits_1km,  bandunits_250m,
     &                           yes_250,
     &                           cube,
     &                           buf_size,
     &                           scan_type, 
     &                           v250_band,
     &                           v1km_band, 
     &                           vis250_band,
     &                           rad_band
     &                         )
      implicit none
      save

c----------------------------------------------------------------------
c!F77
c
c!Description:
c     Routine for reading MODIS level 1B radiances and relfectances
c     and placing in appropriate arrays for processing.
C
c!Input Parameters:
c modfil_L1B_1km Level 1B hdf file array
c fname_1km     Level 1B hdf file name
c modfil_L1B_250m Level 1B hdf file array
c fname_250m    Level 1B hdf file name
c yes_250       Logical flag - true if 250 m file valid
c cube          Scan cube counter
c buf_size      Line and element granule size
c scan_type     Day or night scan
c
c!Output Parameters:
c v250_band     Byte array containing valid flag for a scan of 250m
c               pixels
c v1km_band     Byte array containing valid flag for a scan of 1km
c               pixels
c vis250_band   Array containing the 2 250m resolution visible
c               channel reflectances for a scan cube
c rad_band      Array containing the km resolution channel
c               radiances and brightness temperations for a scan cube
c
c!Revision History:
c
c!Team-Unique Header:
c
c!References and Credits:
c See Cloud Mask ATBD-MOD-06.
c
c!END
c----------------------------------------------------------------------

      include 'global.inc'

c --- parameters
      byte 		v250_band(NX,NY,VIS_BAND)
      byte 		v1km_band(NX,NY,INBAND)
      character*1   	scan_type
      integer 		cube
      integer 		buf_size(2)
      integer 		modfil_L1B_1km(MODFILLEN)
      integer		modfil_L1B_250m(MODFILLEN)
      logical 		yes_250
      real 		rad_band(NX,NY,INBAND)
      real		vis250_band(NX,NY,VIS_BAND)

c --- 1km file swath metadata
      integer		lun_1km
      integer	        datatype_1km
      character*(4)     interleave_1km 
      integer           resolution_1km 
      real		error_1km
      integer	        offset_1km
      integer           samples_1km
      integer           lines_1km  
      integer           bands_1km 
      character*(*)     bandnames_1km(*)
      character*(*)     bandunits_1km(*)

c --- 250m file swath metadata
      integer		lun_250m
      integer	        datatype_250m
      character*(4)     interleave_250m 
      integer           resolution_250m 
      real		error_250m
      integer	        offset_250m
      integer           samples_250m
      integer           lines_250m  
      integer           bands_250m 
      character*(*)     bandnames_250m(*)
      character*(*)     bandunits_250m(*)


c --- external subroutines
      integer 		read_flat_file
      external 		message

c --- internal variables
      byte 		v1km_flag(NPIXEL*SCANS_CUBE)
      byte 		v250m_flag(NPIXEL*SCANS_CUBE*4*4)
      character*1	gain
      character*80      req_bandname
      character*80	req_bandunit
      character*6 	type
      character*6 	band_no
      integer 		ll
      integer 		kk
      integer 		mm
      integer 		nn
      integer 		channel
      integer 		ii
      integer 		jj
      integer 		ElementSize
      integer 		LineSize
      integer		i
      integer		j
      integer		k
      integer		req_resolution
      integer           data_size(2)
      integer           status
      integer		len
      logical 		rtn
      logical		remove_all
      real              v1km_buf(NPIXEL*SCANS_CUBE)
      real              v250m_buf(NPIXEL*SCANS_CUBE*4*4)

c --- Common statement for debug purposes
      integer 		debug
      integer 		h_output
      common / bug / debug, h_output

c --- set inital function status to FAILED
      get_L1b = -1

      remove_all = .TRUE.

c --- Set array sizes needed to read data
      ElementSize = NX
      LineSize = NY

c --- Initialize main radiance array holders
      do 101 ll = 1 , inband
        do 102 mm = 1 , buf_size(2)
          do 103 nn = 1 , buf_size(1)

c --------- 250 m visible channel array holder
            if (ll .le. 2) then
              vis250_band(nn,mm,ll) = -99.0
              v250_band(nn,mm,ll) = -1
            end if

c --------- 1km resolution array holder
            rad_band(nn,mm,ll) =  -99.0
            v1km_band(nn,mm,ll) = -1

  103     continue
  102   continue
  101 continue

c --- debug statement 
      if (debug .gt. 3) then
         write(h_output,'(10x/,''Scan mode is: '',A1,/)') scan_type
      endif


c----------------------------------------------------------------------

c --- Extract the pixels if instrument is in Day mode
      if (scan_type .eq. 'D') then

c ------ channel loop
         Do channel = 1 , inband

c --------- Initialize output variables
            data_size(1) = 0
            data_size(2) = 0

c --------- Initialize input variables
            req_resolution = 1
            write(req_bandname,*) channel
            if( channel.le.19 .or. channel.eq.26 ) then
               req_bandunit = 'ref'
            else
               req_bandunit = 'rad'
            endif

c --------- Read a 1km scan cube-s worth of data out of the granule
            status = read_flat_file(
     &			  	      cube,	 	
     &				      req_resolution,
     &				      req_bandname,
     &				      req_bandunit,
     &				      lun_1km,
     &				      datatype_1km,
     &				      interleave_1km,
     &				      resolution_1km,
     &                                error_1km,
     &				      offset_1km,
     &				      samples_1km,
     &				      lines_1km,
     &				      bands_1km,
     &				      bandnames_1km,
     &				      bandunits_1km,
     &				      ElementSize,
     &				      LineSize,
     &				      v1km_buf,
     &				      v1km_flag,
     &				      data_size(1),
     &				      data_size(2)
     &                               )

             if(status .lt.0 ) then
                write(band_no,'(I6)') channel
                call message('get_L1b',
     &           'Failed to extract 1km L1B data for band '//band_no
     &            ,0, 2 )
             endif

c ---------- Place the data into the 1km holding arrays
             k = 1
             do ii = 1 , data_size(2)
             do jj = 1 , data_size(1)
             rad_band(jj,ii,channel)  = v1km_buf(k)
             v1km_band(jj,ii,channel) = v1km_flag(k)
             k = k+1
             enddo
             enddo

           enddo


c -------- Under certain conditions, change the earth view data validity
c          flag so that the data may be used in the cloud mask.

           do ii = 1, data_size(2)
           do jj = 1, data_size(1)

c ----------- Check for saturated pixels in band 2. Assume saturation
c             if error code is -1 and channel 1 contains valid data.
c             If so, set reflectance to very high value.
              if(v1km_band(jj,ii,2) .eq. -1) then
                 if( 
     &               v1km_band(jj,ii,1) .eq.0          
     &           )then
                     v1km_band(jj,ii,2) = 0          
                     rad_band(jj,ii,2) = 1.2
                 endif
              endif

           enddo
           enddo

c----------------------------------------------------------------------

c ------- check for 250m data
          if (yes_250) then
             Do channel = 1 , 2

c ---------- Initialize output variables
             data_size(1) = 0
             data_size(2) = 0

c ---------- Initialize input variables
             req_resolution = 16
             write(req_bandname,*) channel
             req_bandunit = 'ref'

c ---------- Read a 250m scan cube-s worth of data out of the granule
             status = read_flat_file(
     &			  	      cube,	 	
     &				      req_resolution,
     &				      req_bandname,
     &				      req_bandunit,
     &				      lun_250m,
     &				      datatype_250m,
     &				      interleave_250m,
     &				      resolution_250m,
     &                                error_250m,
     &				      offset_250m,
     &				      samples_250m,
     &				      lines_250m,
     &				      bands_250m,
     &				      bandnames_250m,
     &				      bandunits_250m,
     &				      ElementSize,
     &				      LineSize,
     &				      v250m_buf,
     &				      v250m_flag,
     &				      data_size(1),
     &				      data_size(2)
     &                              )

c ---------- check for error
             if( status.lt.0 ) then
                write(band_no,'(I6)') channel
                call message('get_L1b',
     &         'Failed to extract 250m L1B data for band '// band_no,
     &             0, 0 )
             endif

c ---------- Place the data into the 250m  holding arrays
             k = 1
             do ii = 1, data_size(2), 4
             do jj = 1, data_size(1), 4
               do kk = 0 , 3
                do ll = 0 , 3

                   k = data_size(1) * (ii + kk - 1) + jj + ll
                   vis250_band(jj+ll,ii+kk,channel) = v250m_buf(k)
                   v250_band(jj+ll,ii+kk,channel)   = v250m_flag(k)

                 enddo
                enddo
             enddo
             enddo


          enddo


c ------- Under certain conditions, change the earth view data validity
c         flag so that the data may be used in the cloud mask.

          do ii = 1, data_size(2)
          do jj = 1, data_size(1)

c ------- Check for saturated pixels in band 2. Assume saturation
c         if error code is 65533 and channel 1 contains valid data.

          if(v250_band(jj,ii,2) .eq. -1) then
             if(    
     &           v250_band(jj,ii,1).eq.0             
     &       ) then
                 v250_band(jj,ii,2) = 0             
                 vis250_band(jj,ii,2) = 1.2
             endif
          endif

          enddo
          enddo


c ------- debug statement
          if (debug .gt. 3) then
            write(h_output,'(15x,'' CONTEXT FOR 250 CUBE '',3I10)')cube,
     &              data_size(1), data_size(2)
         write(h_output,'(2x,'' vis250_band 1(.66um)'',/,40(10f10.6/))')
     &              (((vis250_band(i,j,k),i=2780,2789),j=1,40),k=1,1)
         write(h_output,'(2x,'' vis250_band 2(.87um)'',/,40(10f10.6/))')
     &              (((vis250_band(i,j,k),i=2780,2789),j=1,40),k=2,2)
            write(h_output,'(2x,'' v250_band 1(.66um)'',/,40(10i10/))')
     &              (((v250_band(i,j,k),i=2780,2789),j=1,40),k=1,1)
            write(h_output,'(2x,'' v250_band 2(.87um)'',/,40(10i10/))')
     &              (((v250_band(i,j,k),i=2780,2789),j=1,40),k=2,2)
          endif


         endif


c----------------------------------------------------------------------

c --- Extract the pixels if instrument is in Night mode
      else

c ------- only process radiance channels (20-25, 27-36)
          Do channel = (INBAND-IR_BAND+1), INBAND

c ------- Do not extract data if band is 26 (1.38 micron)
          if( channel.ne.26 ) then

c ---------- Initialize output variables
             data_size(1) = 0
             data_size(2) = 0

c ---------- Initialize input variables
             req_resolution = 1
             write(req_bandname,*) channel
             req_bandunit = 'rad'

c ---------- Read a 1km scan cube-s worth of data out of the granule
             status = read_flat_file(
     &	   	  	              cube,	 	
     &				      req_resolution,
     &				      req_bandname,
     &				      req_bandunit,
     &				      lun_1km,
     &				      datatype_1km,
     &				      interleave_1km,
     &				      resolution_1km,
     &                                error_1km,
     &				      offset_1km,
     &				      samples_1km,
     &				      lines_1km,
     &				      bands_1km,
     &				      bandnames_1km,
     &				      bandunits_1km,
     &				      ElementSize,
     &				      LineSize,
     &				      v1km_buf,
     &				      v1km_flag,
     &				      data_size(1),
     &				      data_size(2)
     &                               )

c ---------- check for error
             if( status.lt.0 ) then
                write(band_no,'(I6)') channel
                call message('get_L1b',
     &          'Failed to extract 1km L1B data for band '//band_no,
     &          0, 2 )
             endif

c ---------- Place the data into the 1km holding arrays
             k = 1
             do ii = 1 , data_size(2)
             do jj = 1 , data_size(1)
             rad_band(jj,ii,channel)  = v1km_buf(k)
             v1km_band(jj,ii,channel) = v1km_flag(k)
             k = k+1
             enddo
             enddo

          endif
          enddo


      endif


c --- debug statement 
      if (debug .gt. 3) then
         write(h_output,'(15x,'' CONTEXT FOR 1km CUBE '', 3I10)') cube,
     &                   data_size(1), data_size(2)
         write(h_output,'(15x,'' Pixels 696 - 705'')')
         write(h_output,'(2x,'' rad_band 20 (3.7um)'',/,20(5f10.4/))')
     &       (((rad_band(i,j,k),i=696,705),j=1,10),k=20,20)
         write(h_output,'(2x,'' rad_band 31 (11um)'',/,20(5f10.4/))')
     &       (((rad_band(i,j,k),i=696,705),j=1,10),k=31,31)
         write(h_output,'(2x,'' rad_band 1 (.66um)'',/,20(5f10.4/))')
     &       (((rad_band(i,j,k),i=696,705),j=1,10),k=1,1)
         write(h_output,'(2x,'' rad_band 2 (.87um)'',/,20(5f10.4/))')
     &       (((rad_band(i,j,k),i=696,705),j=1,10),k=2,2)
         write(h_output,'(2x,'' v1km_band 1 (.66um)'',/,20(5i10/))')
     &       (((v1km_band(i,j,k),i=696,705),j=1,10),k=1,1)
         write(h_output,'(2x,'' v1km_band 2 (.87um)'',/,20(5i10/))')
     &       (((v1km_band(i,j,k),i=696,705),j=1,10),k=2,2)
      endif

c ................................................................

c --- set function status to SUCCESS
      get_L1b = 0

      return
      end
