C--------------------------------------------------------------------
C  Copyright (C) 2002,  Space Science and Engineering Center, 
C  University C  of Wisconsin-Madison, Madison WI.
C      
C  This program is free software; you can redistribute it 
C  and/or modify it under the terms of the GNU General 
C  Public License as published by the Free Software Foundation; 
C  either version 2 of the License, or (at your option) any 
C  later version.
C
C  This program is distributed in the hope that it will be 
C  useful, but WITHOUT ANY WARRANTY; without even the implied 
C  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
C  See the  GNU General Public License for more details.
C
C  You should have received a copy of the GNU General Public 
C  License along with this program; if not, write to the Free 
C  Software Foundation, Inc., 59 Temple Place, Suite 330, 
C  Boston, MA  02111-1307 USA
C--------------------------------------------------------------------
C
C
      integer function get_core_metadata(
     &        yes_250,
     &        header_1km,     header_250m,     header_geo, 
     &        nscans,
     &        max_pixel,
     &        datatype_1km,   datatype_250m,   datatype_geo,
     &        interleave_1km, interleave_250m, interleave_geo,
     &        resolution_1km, resolution_250m, resolution_geo,
     &        offset_1km,     offset_250m,     offset_geo,
     &        samples_1km,    samples_250m,    samples_geo,
     &        lines_1km,      lines_250m,      lines_geo,
     &        error_1km,      error_250m,      error_geo,
     &        bands_1km,      bands_250m,      bands_geo,
     &        bandnames_1km,  bandnames_250m,  bandnames_geo,
     &        bandunits_1km,  bandunits_250m,  bandunits_geo
     &                                  ) 
      implicit none
      save

C-----------------------------------------------------------------------
C !F77
C
C !PURPOSE:
C    Extract Core Metadata fields from the MODIS Geolocation product.
C
C !DESCRIPTION:
C    The extracts information from both the the Geolocation file and
C    the the MCF file. The output fields represent the areal and time
C    bounds of the input granule.
C
C !INPUT PARAMETERS:
C    MODFIL_GEO      Geolocation file information array containing
C                    HDF SD_ID (index 1)
C                    VS file identifier (index 2)
C                    File access mode (index 3)
C    MODFIL_LUN      Variable containing the PCF logical
C                    reference number (LRN) to the Geolocation file.
C    MCF_LUN         Variable containing the PCF logical
C                    reference number (LRN) to the metadata
C                    configuration file (MCF)
C
C !OUTPUT PARAMETERS:
C    BEGIN_DATE      Beginning date of granule
C    BEGIN_TIME      Beginning time of granule
C    END_DATE        Ending date of granule
C    END_TIME        Ending time of granule
C    NSCANS	     NUMBER OF SCANS IN THIS IMAGE
C    MAX_PIXEL       MAXIMUM NUMBER OF PIXELS PER SCAN
C
C !REVISION HISTORY
C
C !TEAM-UNIQUE HEADER:
C    Developed by the MODIS Group, CIMSS/SSEC, UW-Madison.
C
C !END
C----------------------------------------------------------------------

      include		'global.inc'

c --- parameters 
      logical		yes_250
      character*(*)	header_1km
      character*(*)	header_250m
      character*(*)	header_geo
      integer		nscans
      integer		max_pixel
      integer		datatype_1km
      integer		datatype_250m
      integer		datatype_geo
      character*(*)	interleave_1km
      character*(*)	interleave_250m
      character*(*)	interleave_geo
      integer		resolution_1km
      integer		resolution_250m
      integer		resolution_geo
      integer		offset_1km
      integer		offset_250m
      integer		offset_geo
      integer		samples_1km
      integer		samples_250m
      integer		samples_geo
      integer		lines_1km 
      integer		lines_250m 
      integer		lines_geo 
      real              error_1km
      real              error_250m
      real              error_geo
      integer		bands_1km
      integer		bands_250m
      integer		bands_geo
      character*(*)	bandnames_1km(*)
      character*(*)	bandnames_250m(*)
      character*(*)	bandnames_geo(*)
      character*(*)	bandunits_1km(*)
      character*(*)	bandunits_250m(*)
      character*(*)	bandunits_geo(*)

c --- external functions
      integer		hdrgetkeydbl
      integer		hdrgetkeystr
      integer		hdrgetkeyint

c --- internal variables
      character*256	filename
      character*80	keyname
      character*80	function
      double precision	d_value
      integer		keyindex
      integer		status
      integer		len
      logical		remove_all

c --- Common Block for debugging code 
      integer			debug
      integer			h_output
      common / bug / debug, h_output


c --- initialize function value 
      get_core_metadata = -1
 
c --- initalize strings
      function = 'get_core_metadata'

      remove_all = .TRUE.


C ---------------- 1KM -------------------

c --- copy file name to internal string
      call strcompress( header_1km, remove_all, len )
      filename = header_1km(1:len)

c --- datatype  
      keyname = 'data type'
      keyindex = 1 
      status=hdrgetkeyint(filename,keyname,keyindex,
     &                    datatype_1km) 
      if( status.lt.0 ) then
         call message( function, 'FAILED - 1km datatype', 0, 3) 
         return
      endif

c --- interleave  
      keyname = 'interleave'
      keyindex = 1 
      status=hdrgetkeystr(filename,keyname,keyindex,
     &                    interleave_1km) 
      if( status.lt.0 ) then
         call message( function, 'FAILED - 1km interleave', 0, 3) 
         return
      endif

c --- resolution  
      keyname = 'resolution'
      keyindex = 1 
      status=hdrgetkeyint(filename,keyname,keyindex,
     &                    resolution_1km) 
      if( status.lt.0 ) resolution_1km = 1

c --- offset  
      keyname = 'offset'
      keyindex = 1 
      status=hdrgetkeyint(filename,keyname,keyindex,
     &                    offset_1km) 
      if( status.lt.0 ) then
         call message( function, 'FAILED - 1km offset', 0, 3) 
         return
      endif

c --- error  
      keyname = 'bad value'
      keyindex = 1 
      status=hdrgetkeydbl(filename,keyname,keyindex,
     &                    d_value) 
      if( status.lt.0 ) then
         call message( function, 'FAILED - 1km error', 0, 3) 
         return
      endif
      error_1km = REAL(d_value)

c --- get the number of scans in the image  
      keyname = 'lines'
      keyindex = 1 
      status=hdrgetkeyint(filename,keyname,keyindex,
     &                    lines_1km) 
      if( status.lt.0 ) then
         call message( function, 'FAILED - 1km lines', 0, 3) 
         return
      endif
      nscans = lines_1km / SCANS_CUBE

c --- get the maximum number of pixels per scan
      keyname = 'samples'
      keyindex = 1 
      status=hdrgetkeyint(filename,keyname,keyindex,
     &                    samples_1km) 
      if( status.lt.0 ) then
         call message( function, 'FAILED - 1km samples', 0, 3) 
         return
      endif
      max_pixel = samples_1km

c --- get the number of bands
      keyname = 'bands'
      keyindex = 1 
      status=hdrgetkeyint(filename,keyname,keyindex,
     &                    bands_1km) 
      if( status.lt.0 ) then
         call message( function, 'FAILED - 1km bands', 0, 3) 
         return
      endif

c --- get the band names and units
      do keyindex = 1, bands_1km
      keyname = 'band names'
      status=hdrgetkeystr(filename,keyname,keyindex,
     &                    bandnames_1km(keyindex)) 
      if( status.lt.0 ) then
         call message(function,'FAILED - 1km band names',0,3) 
         return
      endif
      keyname = 'band units'
      status=hdrgetkeystr(filename,keyname,keyindex,
     &                    bandunits_1km(keyindex)) 
      if( status.lt.0 ) then
         call message(function,'FAILED - 1km band units',0,3) 
         return
      endif
      enddo



C ---------------- 250M -------------------

      if(yes_250 ) then

c ------ copy file name to internal string
         call strcompress( header_250m, remove_all, len )
         filename = header_250m(1:len)

c ------ datatype  
         keyname = 'data type'
         keyindex = 1 
         status=hdrgetkeyint(filename,keyname,keyindex,
     &                       datatype_250m) 
         if( status.lt.0 ) then
            call message( function, 'FAILED - 250m datatype', 0, 3) 
            return
         endif

c ------ interleave  
         keyname = 'interleave'
         keyindex = 1 
         status=hdrgetkeystr(filename,keyname,keyindex,
     &                       interleave_250m) 
         if( status.lt.0 ) then
            call message( function, 'FAILED - 250m interleave', 0, 3) 
            return
         endif

c ------ resolution  
         keyname = 'resolution'
         keyindex = 1 
         status=hdrgetkeyint(filename,keyname,keyindex,
     &                       resolution_250m) 
         if( status.lt.0 ) resolution_250m = 16

c ------ offset  
         keyname = 'offset'
         keyindex = 1 
         status=hdrgetkeyint(filename,keyname,keyindex,
     &                       offset_250m) 
         if( status.lt.0 ) then
            call message( function, 'FAILED - 250m offset', 0, 3) 
            return
         endif

c ------ error  
         keyname = 'bad value'
         keyindex = 1 
         status=hdrgetkeydbl(filename,keyname,keyindex,
     &                       d_value) 
         if( status.lt.0 ) then
            call message( function, 'FAILED - 250m error', 0, 3) 
            return
         endif
         error_250m = REAL(d_value)

c ------ lines 
         keyname = 'lines'
         keyindex = 1 
         status=hdrgetkeyint(filename,keyname,keyindex,lines_250m) 
         if( status.lt.0 ) then
            call message( function, 'FAILED - 250m lines', 0, 3) 
            return
         endif

c ------ samples
         keyname = 'samples'
         keyindex = 1 
         status=hdrgetkeyint(filename,keyname,keyindex,samples_250m) 
         if( status.lt.0 ) then
            call message( function, 'FAILED - 250m samples', 0, 3) 
            return
         endif

c ------ bands
         keyname = 'bands'
         keyindex = 1 
         status=hdrgetkeyint(filename,keyname,keyindex,bands_250m) 
         if( status.lt.0 ) then
            call message( function, 'FAILED - 250m bands', 0, 3) 
            return
         endif

c ------ band names and units
         do keyindex = 1, bands_250m
         keyname = 'band names'
         status=hdrgetkeystr(filename,keyname,keyindex,
     &                       bandnames_250m(keyindex)) 
         if( status.lt.0 ) then
            call message(function,'FAILED - 250m band names',0,3) 
            return
         endif
         keyname = 'band units'
         status=hdrgetkeystr(filename,keyname,keyindex,
     &                       bandunits_250m(keyindex)) 
         if( status.lt.0 ) then
            call message(function,'FAILED - 250m band units',0,3) 
            return
         endif
         enddo

      endif


C ---------------- GEO -------------------

c --- copy file name to internal string
      call strcompress( header_geo, remove_all, len )
      filename = header_geo(1:len)

c --- datatype  
      keyname = 'data type'
      keyindex = 1 
      status=hdrgetkeyint(filename,keyname,keyindex,
     &                    datatype_geo) 
      if( status.lt.0 ) then
         call message( function, 'FAILED - Geo datatype', 0, 3) 
         return
      endif

c --- interleave  
      keyname = 'interleave'
      keyindex = 1 
      status=hdrgetkeystr(filename,keyname,keyindex,
     &                    interleave_geo) 
      if( status.lt.0 ) then
         call message( function, 'FAILED - Geo interleave', 0, 3) 
         return
      endif

c --- resolution  
      keyname = 'resolution'
      keyindex = 1 
      status=hdrgetkeyint(filename,keyname,keyindex,
     &                    resolution_geo) 
      if( status.lt.0 ) resolution_geo = 1

c --- offset  
      keyname = 'offset'
      keyindex = 1 
      status=hdrgetkeyint(filename,keyname,keyindex,
     &                    offset_geo) 
      if( status.lt.0 ) then
         call message( function, 'FAILED - Geo offset', 0, 3) 
         return
      endif

c --- error  
      keyname = 'bad value'
      keyindex = 1 
      status=hdrgetkeydbl(filename,keyname,keyindex,
     &                    d_value) 
      if( status.lt.0 ) then
         call message( function, 'FAILED - Geo error', 0, 3) 
         return
      endif
      error_geo = REAL(d_value)

c --- get the number of scans in the image  
      keyname = 'lines'
      keyindex = 1 
      status=hdrgetkeyint(filename,keyname,keyindex,
     &                    lines_geo) 
      if( status.lt.0 ) then
         call message( function, 'FAILED - Geo lines', 0, 3) 
         return
      endif

c --- get the maximum number of pixels per scan
      keyname = 'samples'
      keyindex = 1 
      status=hdrgetkeyint(filename,keyname,keyindex,
     &                    samples_geo) 
      if( status.lt.0 ) then
         call message( function, 'FAILED - Geo samples', 0, 3) 
         return
      endif

c --- get the number of bands
      keyname = 'bands'
      keyindex = 1 
      status=hdrgetkeyint(filename,keyname,keyindex,
     &                    bands_geo) 
      if( status.lt.0 ) then
         call message( function, 'FAILED - Geo bands', 0, 3) 
         return
      endif

c --- get the band names and units
      do keyindex = 1, bands_geo
      keyname = 'band names'
      status=hdrgetkeystr(filename,keyname,keyindex,
     &                    bandnames_geo(keyindex)) 
      if( status.lt.0 ) then
         call message(function,'FAILED - Geo band names',status,3) 
         return
      endif
      keyname = 'band units'
      status=hdrgetkeystr(filename,keyname,keyindex,
     &                    bandunits_geo(keyindex)) 
      if( status.lt.0 ) then
         bandunits_geo(keyindex) = ' '
      endif
      enddo



c --- Write debug output if required
      if (debug.gt.0) then

        WRITE(h_output , '(10x,'' Processing dimensions '')' )
        WRITE( h_output ,
     &     '(10x,'' Scans in Image    Pixels per Scan'','//
     &     '/,5x,I15,5x,I15/)') 
     &     nscans, max_pixel

      endif

      get_core_metadata = 0
      return
      end
