C--------------------------------------------------------------------
C  Copyright (C) 2002,  Space Science and Engineering Center, 
C  University C  of Wisconsin-Madison, Madison WI.
C      
C  This program is free software; you can redistribute it 
C  and/or modify it under the terms of the GNU General 
C  Public License as published by the Free Software Foundation; 
C  either version 2 of the License, or (at your option) any 
C  later version.
C
C  This program is distributed in the hope that it will be 
C  useful, but WITHOUT ANY WARRANTY; without even the implied 
C  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
C  See the  GNU General Public License for more details.
C
C  You should have received a copy of the GNU General Public 
C  License along with this program; if not, write to the Free 
C  Software Foundation, Inc., 59 Temple Place, Suite 330, 
C  Boston, MA  02111-1307 USA
C--------------------------------------------------------------------
C
C
      integer function put_mod35(
     &                            mask_lun,
     &                            hdr_lun,
     &                            qa_lun,
     &                            qahdr_lun,
     &                            s_pixels,
     &                            nlin,
     &                            klin,
     &                            lines_in_edge,
     &                            mask_array,
     &                            qa_array
     &                          )

      implicit none 
      save

C!F77------------------------------------------------------------------
C
C!DESCRIPTION:  Write Scientific Data Set (SDS) for cloud
C               mask algorithm. The process is complicated
c               by the way the cloud mask is processed. The
c               method used is a NLCNTX x necntx sliding box,
c               so more than one line of data is being used
c               at a time.  This program adds a line of data
c               to the holding array, then writes it out
c               once the tenth line has been filled.
c
C!INPUT PARAMETERS:
c s_pixels      Context of pixels in current scan
C nlin          Total number of lines to process
c klin          Counts number of lines output to bit file
c lines_in_edge Number of lines outside of processing region
C modfil_mod35 	Array used to reference MODIS HDF file
C mask_array    Byte array containing line of cloud mask output
C qa_array      Byte array containing line of cloud mask qa output
C
C!OUTPUT PARAMETERS:
c None
C
C!REVISION HISTORY:
C
C!TEAM-UNIQUE HEADER:
C
C!REFERENCES and CREDITS:
C
C!DESIGN NOTES:
C       Functions:
C          CRMAR                           (mapi.inc)
C          PMAR                            (mapi.inc)
C
C       Named Constant:
C          MAPIOK                          (mapi.inc)
c          MODFILLEN                       (mapi.inc)
C
C!END------------------------------------------------------------------

      INCLUDE 'global.inc'

c --- constants
      integer 		MASK_DIM
      parameter		(MASK_DIM = 6)

      integer 		QA_DIM
      parameter		(QA_DIM = 10)

      integer  		LINES_PER_WRITE
      parameter		(LINES_PER_WRITE = SCANS_CUBE)

c --- parameters 
      integer		mask_lun
      integer		hdr_lun
      integer		qa_lun
      integer		qahdr_lun
      integer 		nlin
      integer 		klin
      integer 		lines_in_edge
      integer		s_pixels(NLCNTX)
      byte 		mask_array(NPIXEL,MASK_DIM)
      byte 		qa_array(NPIXEL,QA_DIM)
 
c --- external function
      integer		write_flat_file

c --- internal variables
      character*255     mask_file_desc
      character*255     qamask_file_desc
      integer 		buf_line_counter
      integer 		line_counter
      integer 		ipxl
      integer 		status
      integer 		ibyte
      integer		nele
      integer		i
      integer		iline
      integer		buf_offset
      integer		buf_index
      integer		scan
      integer		fil_lun
      integer		hdr_flg
      integer		out_maxsamples
      integer		out_maxlines
      integer		out_samples
      integer		out_lines
      logical           init
      byte 		sstore1(NPIXEL*SCANS_CUBE*MASK_DIM)
      byte 		sstore2(NPIXEL*SCANS_CUBE*QA_DIM)
      byte		out_flag(NPIXEL*SCANS_CUBE*QA_DIM)

      character*4	req_interleave,    fil_interleave
      character*80	req_bandnames(10), fil_bandnames(10)
      character*80	req_bandunits(10), fil_bandunits(10)
      integer		req_resolution,    fil_resolution
      integer		req_datatype,      fil_datatype
      integer		req_offset,        fil_offset
      integer		req_samples,       fil_samples
      integer		req_lines,         fil_lines
      integer		req_bands,         fil_bands
      byte		req_error,         fil_error

c --- Common block used for writing debug output statements
      integer		debug
      integer 		h_output
      common / bug / debug, h_output

c --- initialize line counter
      data 		line_counter /0/
      data		scan /0/
      data		init /.true./

      
c --- debug statement
      if( debug.gt.0) then
        write(h_output,'(10x/,''Processing subroutine mod35_out '',/)')
      endif

c --- initialize the flag array (unused by Cloud Mask)
      if( line_counter.eq.0 ) then
         do i = 1, (NPIXEL*SCANS_CUBE*MASK_DIM)
         sstore1(i) = 0
         enddo
         do i = 1, (NPIXEL*SCANS_CUBE*QA_DIM)
         sstore2(i) = 0
         out_flag(i) = 0
         enddo
      endif 

c --- Initialization
c     Need to know how many pixels to store for this scan
      if (klin .le. lines_in_edge) then
         i = klin
      else if (klin .gt. nlin-lines_in_edge) then
         i = NLCNTX
      else
         i = ((NLCNTX-1) / 2) + 1
      endif
      hdr_flg = 0

c --- increment the line counter
      line_counter = line_counter+1

c --- Enter hdf file array dimensions
      nele = s_pixels(i)

c --- Determine current line in work buffer (buf), and transfer
c     data from mask_array to work buffer.
      buf_line_counter = mod(line_counter-1,LINES_PER_WRITE) + 1

c --- debug statement 
      if( debug.gt.0) then
        write(h_output,'(1x,''lc1 lc2 nele nlin'',4i10)') line_counter,
     +          buf_line_counter,nele,nlin
      endif

c --- fill the mask and quality array
      iline = buf_line_counter
      do ibyte = 1 , MASK_DIM
          buf_offset = (ibyte-1)*nele*LINES_PER_WRITE
          do ipxl = 1 , nele
            buf_index = buf_offset + (iline-1) * nele + ipxl
            sstore1(buf_index) = mask_array(ipxl,ibyte)
          end do
      end do

      do ibyte = 1 , QA_DIM
          buf_offset = (ibyte-1)*nele*LINES_PER_WRITE
          do ipxl = 1 , nele
            buf_index = buf_offset + (iline-1) * nele + ipxl
            sstore2(buf_index) = qa_array(ipxl,ibyte)
          end do
      end do

c --- if the number of lines in buffer is 10, then write out
      if( buf_line_counter.eq.lines_per_write .or.
     &    line_counter.eq.nlin                     ) then
  

          if (init) then
              hdr_flg = 1 
              mask_file_desc = 'MODIS Direct Broadcast Cloud Mask'
              qamask_file_desc = 'MODIS Direct Broadcast Quality ' // 
     &                           'Assurance'
              init = .false.
          endif

c ------ increment the scan counter
         scan = scan+1

c ------ set cloud mask request parameters
         req_samples      = nele 		! bytes per line
         req_lines        = buf_line_counter	! number of lines

c ------ set cloud mask file parameters
         fil_lun          = mask_lun		! logical unit of mask file
         fil_datatype     = 1			! 8 bit byte
         fil_interleave   = 'bsq'		! band sequential
         fil_resolution   = 1			! 1km resolution
         fil_error        = -1		        ! internal error code
         fil_offset       = 0			! size of file header
         fil_samples      = nele		! bytes per line
         fil_lines        = nlin 		! number of lines
         fil_bands        = MASK_DIM 		! number of bands
         do i = 1, fil_bands
            write(fil_bandnames(i),*) 'byte ',i !  list of band names
            fil_bandunits(i) = 'flg'		! list of band units
         enddo

         out_maxsamples   = NPIXEL * MASK_DIM   ! maximum element size of array
         out_maxlines     = SCANS_CUBE          ! maximum line size of array

c ------ write the mask array to a flat output file   
         status = write_flat_file( 
     &			    	   scan,	 	
     &				   req_samples,
     &				   req_lines,
     &				   fil_lun,
     &                             hdr_flg,
     &                             hdr_lun,
     &                             mask_file_desc,
     &				   fil_datatype,
     &				   fil_interleave,
     &				   fil_resolution,
     &                             fil_error,
     &				   fil_offset,
     &				   fil_samples,
     &				   fil_lines,
     &				   fil_bands,
     &				   fil_bandnames,
     &				   fil_bandunits,
     &				   out_maxsamples,
     &				   out_maxlines,
     &				   sstore1,
     &				   out_flag,
     &				   out_samples,
     &				   out_lines
     &                           )

         if( status.lt.0 ) then
            call message( 'put_mod35',
     &                    'Could not write product bits to output file',
     &                    0, 3 )
         endif


c ------ set qa request parameters
         req_datatype     = 1			! 8 bit byte
         req_interleave   = 'bsq'               ! band sequential
         req_resolution   = 1 			! 1km resolution
         req_error        = -1			! internal error code
         req_offset       = 0			! size of file header
         req_samples      = nele		! bytes per line
         req_lines        = buf_line_counter 	! number of lines
         req_bands        = QA_DIM 		! number of bands
         do i = 1, req_bands 
            write(req_bandnames(i),*) i  	! list of band names
            req_bandunits(i) = 'flg'		! list of band units
         enddo

c ------ set qa file parameters
         fil_lun          = qa_lun		! logical unit of qa file
         fil_datatype     = 1			! 8 bit byte
         fil_interleave   = 'bsq'		! band sequential
         fil_resolution   = 1			! 1km resolution
         fil_error        = -1			! internal error code
         fil_offset       = 0			! size of file header
         fil_samples      = nele		! bytes per line
         fil_lines        = nlin		! number of lines
         fil_bands        = QA_DIM 		! number of bands
         do i = 1, fil_bands 
            write(fil_bandnames(i),*) 'byte ',i !  list of band names
            fil_bandunits(i) = 'flg'		! list of band units
         enddo

         out_maxsamples   = NPIXEL * QA_DIM     ! maximum element size of array
         out_maxlines     = SCANS_CUBE          ! maximum line size of array

c ------ write the qa array to a flat output file   
         status = write_flat_file( 
     &			    	   scan,	 	
     &				   req_samples,
     &				   req_lines,
     &				   fil_lun,
     &                             hdr_flg,
     &                             qahdr_lun,
     &                             qamask_file_desc,
     &				   fil_datatype,
     &				   fil_interleave,
     &				   fil_resolution,
     &                             fil_error,
     &				   fil_offset,
     &				   fil_samples,
     &				   fil_lines,
     &				   fil_bands,
     &				   fil_bandnames,
     &				   fil_bandunits,
     &				   out_maxsamples,
     &				   out_maxlines,
     &				   sstore2,
     &				   out_flag,
     &				   out_samples,
     &				   out_lines
     &                           )

         if( status.lt.0 ) then
            call message( 'put_mod35',
     &                    'Could not write qa bits to output file',
     &                    0, 3 )
         endif

c ------ reset the line counter
         line_counter = 0

      endif     
      hdr_flg = 0

c --- set function status to SUCCESS
      put_mod35 = 0

      return
      end
