C--------------------------------------------------------------------
C  Copyright (C) 2002,  Space Science and Engineering Center, 
C  University C  of Wisconsin-Madison, Madison WI.
C      
C  This program is free software; you can redistribute it 
C  and/or modify it under the terms of the GNU General 
C  Public License as published by the Free Software Foundation; 
C  either version 2 of the License, or (at your option) any 
C  later version.
C
C  This program is distributed in the hope that it will be 
C  useful, but WITHOUT ANY WARRANTY; without even the implied 
C  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
C  See the  GNU General Public License for more details.
C
C  You should have received a copy of the GNU General Public 
C  License along with this program; if not, write to the Free 
C  Software Foundation, Inc., 59 Temple Place, Suite 330, 
C  Boston, MA  02111-1307 USA
C--------------------------------------------------------------------
C
C
      subroutine PolarDay_desert(pxldat,vza,visusd,cirrus_vis,hi_elev,
     +                           testbits,qa_bits,nmtests,confdnc)
 
      implicit none
      save

c--------------------------------------------------------------------
C!F77 
c
c!Description:
c      Routine for performing clear sky tests over polar desert 
c      surfaces during daylight hours.
c
c      For each threshold, a "low confidence" and a "high confidence"
c      cutoff value is defined ('locut','hicut').  Confidence is
c      0 % beyond 'locut' and 100% beyond 'hicut'.  Please note
c      that 'locut' may be arithmetically greater that 'hicut'.
c      For tests which use a range of values (like the visible ratio
c      test), two low cutoffs and two high cutoffs are required, one
c      pair for each end of the range.  In this case, the two pairs
c      must be entered in numeric order (lower pair in index 1 of
c      'locut' and 'hicut').
c
c      For daytime polar desert the groups are:
c          Group 1: High thick cloud
c                   6.75 micron bt test 
c
c          Group 2: Mainly Low cloud - thick
c                   11-4 micron bt tests
c                   11-12 Thin cirrus tests
c        
c          Group 3: Thick cloud
c                   .87 micron reflectance test (masv87)
c 
c          Group 4: Thin cirrus test
c                   1.38 micron reflectance test 
c
c!Input Parameters:
c real    pxldat   Array containing reflectance or brightness temperatures
c                  for all bands for a single pixel
c real    vza      Current pixel viewing angle
c logical visusd   Logical variable indicating whether vis data used 
c logical hi_elev  Flag indicating elevation > 2000 meters
c
c!Output Parameters:
c byte    testbits six byte integer containing bit results
c byte    qa_bits  ten byte array containing QA bit results
c integer nmtests  Counts number of inidividual tests applied
c real    confdnc  product of all applied individual confidences
c logical cirrus_vis Logical variable flagging thin cirrus contaminated
c                scenes in the visible
c
c!Revision History:
c
c!Team-Unique Header:
c
c!References and Credits:
c See Cloud Mask ATBD-MOD-06.
c 1996/4/3 13:55:25
c K. Strabala (kathys@ssec.wisc.edu)
c Original version.
c
c!Design Notes:
c
c   Externals:
c      Subroutines:  
c      conf_test_2val,conftest,tview,set_bit,clear_bit
c      set_qa_bit
c
c!End
c----------------------------------------------------------------------

      include 'global.inc'
      include 'PolarDay_desert_thr.inc'


c ... scalar arguments ..
      real confdnc,vza
      integer nmtests
      logical visusd,cirrus_vis,hi_elev
c ...
c ... array arguments ..
      real pxldat(inband)
      byte testbits(6),qa_bits(10)
c ...
c ... local scalars ..
      real c2,c3,c4,c5,cosvza,dfthrsh,diftemp,dtr,mas11_4,masdf1,
     +     masir11,masir12,masir13,masir4,masir8,masv162,masv188,
     +     masir3,masv95,masv66,masv88,pi,schi,
     +     masv55,masir65,c7,cmin1,cmin2,cmin3,cmin4,pre_confdnc,
     +     Rel_equality_EPS,groups,fac,locut,hicut
      parameter (Rel_equality_EPS = 0.000001)
      integer nptests,kk,debug,h_output
c ...
c ... local arrays ..
      real hicuta(2),locuta(2),midpta(2)
      integer ngtests(4)
c ...
c ... external subroutines ..
      external conf_test,conf_test_2val,tview,set_bit,
     *         clear_bit,set_qa_bit

c ... Common statement for debug purposes
      common / bug / debug, h_output
c ...
c ... initialize variables
      pi = acos(-1.0)
      dtr = pi/180.0

c ... nmtests counts the number of tests applied to this pixel
      nmtests = 0

c ... nptests counts the number of tests passed
      nptests = 0
c ... set confidence to 1.0 to begin with
      confdnc = 1.0
c ... place band values into individual variables for easy
c ... identification
      masv55 = pxldat(4)
      masv66 = pxldat(1)
      masv88 = pxldat(2)
      masv95 = pxldat(19)
      masv188 = pxldat(26)
      masv162 = pxldat(6)
      masir3 = pxldat(20)
      masir4 = pxldat(22)
      masir65 = pxldat(27)
      masir8 = pxldat(29)
      masir11 = pxldat(31)
      masir12 = pxldat(32)
      masir13 = pxldat(35)

c ...
      masdf1 = 0.0
      cosvza = 0.0
      schi = 0.0
      diftemp = 0.0
      dfthrsh = 0.0
      mas11_4 = 0.0

c ... the c suffix variables represent individual test confidences
      c2 = 0.0
      c3 = 0.0
      c4 = 0.0
      c5 = 0.0
      c7 = 0.0
      cmin1 = 1.0
      cmin2 = 1.0
      cmin3 = 1.0
      cmin4 = 1.0

c ... initialize group number holder
      do 10 kk = 1 , 4
         ngtests(kk) = 0
  10  continue

c ... debug statement ............................................
      if (debug .gt. 0) then
        write(h_output,'(10x/,''Subroutine PolarDay_desert '',/)')
      endif
c ................................................................

 
c ... perform tests.  note that some tests are not used
c ... in sun glint conditions.
c
C     **** GROUP 1 TESTS *************************************

c     H20 vapor channel (6.7 micron) high cloud test
      if (nint(masir65) .ne. nint(bad_data)) then
        nmtests = nmtests + 1
        call set_qa_bit(qa_bits,15)
        if (masir65 .gt. pdsh20(2)) then
          call set_bit(testbits,15)
          nptests = nptests + 1
        end if
        call conf_test(masir65,pdsh20(1),pdsh20(3),pdsh20(4),
     *                 pdsh20(2),1,c2)
        cmin1 = min(cmin1,c2)
        ngtests(1) = ngtests(1) + 1
      endif

c ... debug statement ............................................
      if (debug .gt. 0) then
        write(h_output,'(1x,''masir65: '',5f10.2)') masir65,pdsh20(1),
     +          pdsh20(2),pdsh20(3),pdsh20(4)
      endif
c ................................................................
c     *****  END OF GROUP 1 TESTS  ***************************
c
c
c
c     ****  GROUP 2 TESTS  ***********************************
c ... 11-12um brightness temperature difference test
c ... for thin cirrus).
      if (nint(masir11) .ne. nint(bad_data)
     +                 .and.
     +    nint(masir12) .ne. nint(bad_data) 
     +                  .and. 
     +              vza .gt. 0.0) then
        masdf1 = masir11 - masir12
c ...   added apollo viewing angle/av4t regressed threshold.
c ...   calculate secant of viewing zenith angle.
        cosvza = cos(vza*dtr)
        if (abs(cosvza).gt.Rel_equality_EPS) then
          schi = 1.0/cosvza
        else
          schi = 99.0
        end if

c ...   interpolate look-up table values of 11 - 12 micron bt
c ...   difference thresholds (function of viewing zenith
c ...   and 11 micron brightness temperature).
        call tview(1,schi,masir11,diftemp)

c ...   if a threshold was determined by apollo, then use this
c ...   as the thin cirrus test, otherwise use a standard threshold
        if (diftemp.lt.0.1 .or. abs(schi-99.0).lt.0.0001) then
          dfthrsh = pds11_12hi(2)
        else
          dfthrsh = diftemp
        end if

c...    Set flags if test passed
        nmtests = nmtests + 1
        call set_qa_bit(qa_bits,18)
        if (masdf1.le.dfthrsh) then
          call set_bit(testbits,18)
          nptests = nptests + 1
        end if
        locut = dfthrsh + 0.5
        hicut = dfthrsh - 1.25
        call conf_test(masdf1,locut,hicut,1.0,dfthrsh,1,c3)
        cmin2 = min(cmin2,c3)
        ngtests(2) = ngtests(2) + 1
      endif

c ... debug statement ............................................
      if (debug .gt. 0) then
        write(h_output,'(1x,''APOLLO masdf1: '',4f10.2)') masdf1,
     +          dfthrsh,locut,hicut
      endif
c ................................................................

  
c ... 11 minus 4 micron BTDIF fog and low cloud test.
      if (visusd) then
        if (nint(masir11) .ne. nint(bad_data)
     +                   .and.
     +      nint(masir4) .ne.  nint(bad_data)) then
          if(masir11 .le. 320.0) then
            nmtests = nmtests + 1
            mas11_4 = masir11 - masir4
            call set_qa_bit(qa_bits,19)
            if(mas11_4.ge.pds11_4lo(2) .and. 
     &         mas11_4.le.pds11_4hi(2)       )then
              call set_bit(testbits,19)
              nptests = nptests + 1
            end if
            locuta(1) = pds11_4lo(1)
            locuta(2) = pds11_4hi(1)
            hicuta(1) = pds11_4lo(3)
            hicuta(2) = pds11_4hi(3)
            midpta(1) = pds11_4lo(2)
            midpta(2) = pds11_4hi(2)
  
            call conf_test_2val(mas11_4,locuta,hicuta,1.0,midpta,2,c4)
            cmin2 = min(cmin2,c4)
            ngtests(2) = ngtests(2) + 1
          endif
        end if

c ..... debug statement ............................................
        if (debug .gt. 0) then
           write(h_output,'(1x,''mas11_4a: '',5f10.2)')
     &     mas11_4,pds11_4lo(1),
     &     pds11_4lo(2),pds11_4lo(3),pds11_4lo(4)
           write(h_output,'(1x,''mas11_4b: '',5f10.2)')
     &     mas11_4,pds11_4hi(1),
     &     pds11_4hi(2),pds11_4hi(3),pds11_4hi(4)
        endif
c ..................................................................
      endif


c *******     END OF GROUP 2 TESTS ****************************
c
c
c
c ********  START OF GROUP 3 TESTS ****************************
c ... visible (.88 micron) reflectance threshold test.
      if (visusd) then
        if (nint(masv88) .ne. nint(bad_data)) then
          nmtests = nmtests + 1
          call set_qa_bit(qa_bits,20)
          if (masv88 .le. pdsref2(2)) then
            call set_bit(testbits,20)
            nptests = nptests + 1
          end if
          call conf_test(masv88,pdsref2(1),pdsref2(3),pdsref2(4),
     +                  pdsref2(2),1,c5)
          cmin3 = min(cmin3,c5)
          ngtests(3) = ngtests(3) + 1
        end if

c  ...  debug statement ............................................
        if (debug .gt. 0) then
          write(h_output,'(1x,''masv88: '',5f10.2)') masv88,pdsref2(1),
     +            pdsref2(2),pdsref2(3),pdsref2(4)
        endif
c ..................................................................
      endif
c ******       END OF GROUP 3 TESTS   ****************************
 
 
 
 
c ***********   START OF GROUP 4 TESTS  *************************
c ... near infrared high cloud test
      if (visusd .and. (.not. hi_elev) ) then
        if (nint(masv188) .ne. nint(bad_data)) then
          nmtests = nmtests + 1
          call set_qa_bit(qa_bits,16)
          if (masv188 .le. pdsref3(2)) then
            call set_bit(testbits,16)
            nptests = nptests + 1
          end if
          call conf_test(masv188,pdsref3(1),pdsref3(3),pdsref3(4),
     +                  pdsref3(2),1,c7)
          cmin4 = min(cmin4,c7)
          ngtests(4) = ngtests(4) + 1
        end if
 
c ...   debug statement ............................................
        if (debug .gt. 0) then
           write(h_output,'(1x,''masv188: '',6f10.4)')
     &     masv188,pdsref3(1),
     &     pdsref3(2),pdsref3(3),pdsref3(4)
        endif
c ................................................................
      endif
c ************   END OF GROUP 4 TESTS   ****************************
 
 
 
c     Check to see if thin cirrus bit should be set
c     This is the case where the reflectances in the 1.38 are low
c     and are not above a .66 micron thresholds either, to prevent
c     low bright clouds from being flagged as cirrus.
      if (nint(masv188) .ne. nint(bad_data)) then
        call set_qa_bit(qa_bits,9)
        if (masv188 .lt. pdsref3(2) .and. masv188 .ge. pdsref3(3)) then
           if (masv88 .lt. pdsref2(1)) then
              call clear_bit(testbits,9)
              cirrus_vis = .true.
           endif
        endif
      endif
c
c     Determine final confidence based on group values
      pre_confdnc = cmin1 * cmin2 * cmin3 * cmin4

c     Next, make sure you have all groups covered
      groups = 0
      do kk = 1,4
        if(ngtests(kk) .gt. 0) then
          groups = groups + 1.0
        end if
      enddo
      if (groups .gt. 0) fac = 1.0 / groups
c     Find final pixel confidence as nth root of group tests
      confdnc = pre_confdnc**fac

c ... debug statement ............................................
      if (debug .gt. 0) then
        write(h_output,'(1x,''tests '',6i10)') nmtests,nptests,ngtests
        write(h_output,'(1x,''confdnc '',7f8.5/,4f8.5)') c2,c3,c4,c5,
     +         c7,cmin1,cmin2,cmin3,cmin4,fac,confdnc
      endif
c ................................................................

      return
      end
